# -*- coding: utf-8 -*-
#CPE 1704 TKS
"""
Module implementing MainWindow GUI, anselA
"""
import glob
import math
import operator
import os
import sys
import webbrowser
from subprocess import call

from PIL import Image
from PIL import ImageDraw
from PIL import ImageEnhance
from PyQt4 import QtCore as QtCore
from PyQt4 import QtGui as QtGui 
from PyQt4.QtCore import pyqtSignature
from PyQt4.QtGui import QMainWindow

import imports.EXIF
from core.browser import *
from core.core import *
from core.buttons import *
from core.files import *
from filters.bw import *
from filters.colors import *
from filters.histogram import *
from filters.imageExif import *
from filters.levels import *
from filters.rotate import *
from filters.toolbox import *
from filters.simple import *
from ui.Ui_mainWindow import Ui_MainWindow
from filters.toolbox import pil2qpixmap


class MainWindow(QMainWindow, Ui_MainWindow,  ParamValues):
    """
    Main Window Class for AnselA
    """
    core=ParamValues()

    #######################################
    #   Initial 
    #######################################

    def __init__(self, parent = None):
        """
        Constructor
        """
        try:
            QMainWindow.__init__(self, parent)
            self.setupUi(self)

        except:
            print "Error: init"
            return
            
    def postShow(self):
        try:
            self.WaitOverlay("Starting...")
            self.FullScreenToggle(True, 2)
            self.tabStrip.setCurrentIndex(0)
            
            self.photo.setMouseTracking= True
            self.WaitOverlayHide()
            populatePhotoList(self)
            self.listPhotositemLoad(self.listPhotos.item(0))

            FileImageOpen(self,  self.core.curImageLoc)
            self.ShowApply(False)

            self.core.plotIm=Image.open("/opt/anselA/ui/resources/graph.png") 

            
        except:
            print "Error: postShow"
            


    #######################################
    #   Display functions
    #######################################

    @pyqtSignature("Bool")
    def ShowApply(self, Show,  First = False):
        if self.core.ApplyVisible <> Show:
            self.core.ApplyVisible = Show
            self.btnApplyUndo.setText(":"+str(self.core.undoCount))
            if Show:
                self.btnApply.setVisible(True)
            else:
                self.btnApply.setVisible(False)
            if First:
                self.btnApplyUndo.setVisible(False)
            else:
                self.btnApplyUndo.setVisible(True)
        
    @pyqtSignature("Bool, Image")
    def showUpdateLabel(self, show,  thumb):
        self.ShowApply(True)
        self.lockUpdating = show
        if (thumb == True):
            if (show == True):
                self.labWaitThumb.setGeometry(self.photo_Thumb.geometry())
            else:
                self.labWaitThumb.setGeometry(QtCore.QRect(0, 0, 0, 0))
        else:
            if (show == True):
                self.FullScreenToggle(True)
                self.WaitOverlay("Updating Photo...",  "Large images may take a few seconds to process.")
            else:
                self.WaitOverlayHide()
                self.FullScreenToggle(False)
        
        self.repaint()

    #######################################
    #   Color Sliders
    #######################################


    @pyqtSignature("int")
    def on_cSlide_B_valueChanged(self, value):
        if (self.core.valB <> value):
            self.core.valB = cSlide_SetValue(self, value,  self.core.valB,  self.lcdB, self.cSlide_B)
        
    def on_btnSlide_B_up_pressed(self):
        self.core.valB = cSlide_ShiftValue(self,  1,  self.core.valB,  self.lcdB, self.cSlide_B)
        
    def on_btnSlide_B_dn_pressed(self):
        self.core.valB = cSlide_ShiftValue(self, -1,  self.core.valB,  self.lcdB, self.cSlide_B)

    @pyqtSignature("int")
    def on_cSlide_R_valueChanged(self, value):
        """
        Set Red Value
        """
        if (self.core.valR <> value):
            self.core.valR = cSlide_SetValue(self, value,  self.core.valR,  self.lcdR, self.cSlide_R)
        
    def on_btnSlide_R_dn_pressed(self):
        self.core.valR = cSlide_ShiftValue(self,  -1,  self.core.valR,  self.lcdR, self.cSlide_R)
        
    def on_btnSlide_R_up_pressed(self):
        self.core.valR = cSlide_ShiftValue(self, 1,  self.core.valR,  self.lcdR, self.cSlide_R)       
    
    @pyqtSignature("int")
    def on_cSlide_G_valueChanged(self, value):
        """
        Set Green Value
        """
        if (self.core.valG <> value):
            self.core.valG = cSlide_SetValue(self, value,  self.core.valG,  self.lcdG, self.cSlide_G)
        
    def on_btnSlide_G_up_pressed(self):
        self.core.valG = cSlide_ShiftValue(self,  1,  self.core.valG,  self.lcdG, self.cSlide_G)
        
    def on_btnSlide_G_dn_pressed(self):
        self.core.valG = cSlide_ShiftValue(self, -1,  self.core.valG,  self.lcdG, self.cSlide_G)
    

  
    #######################################
    #   Level Applys
    #######################################
  
    def on_cSlide_Mi_valueChanged(self, value):
        if (self.core.valMi <> value):
            self.core.valMi = cSlide_SetValue(self, value,  self.core.valMi,  self.lcdMi_2, self.cSlide_Mi)
        
    def on_btnSlide_Mid_dn_pressed(self):
        self.core.valMi= cSlide_ShiftValue(self,  -1,  self.core.valMi,  self.lcdMi_2, self.cSlide_Mi)
            
    def on_btnSlide_Mid_up_pressed(self):
        self.core.valMi = cSlide_ShiftValue(self,  1,  self.core.valMi,  self.lcdMi_2, self.cSlide_Mi)
  
  
    def on_cSlide_Hi_valueChanged(self, value):
        if (self.core.valHi <> value):
            self.core.valHi = cSlide_SetValue(self, value,  self.core.valHi,  self.lcdHi, self.cSlide_Hi)

    def on_btnSlide_H_dn_pressed(self):
        self.core.valHi = cSlide_ShiftValue(self,  -1,  self.core.valHi,  self.lcdHi, self.cSlide_Hi)
        
    def on_btnSlide_H_up_pressed(self):
        self.core.valHi = cSlide_ShiftValue(self,   1,  self.core.valHi,  self.lcdHi, self.cSlide_Hi)
    
    
    def on_cSlide_Sw_valueChanged(self, value):
        if (self.core.valSw <> value):
            self.core.valSw = cSlide_SetValue(self, value,  self.core.valSw,  self.lcdSw, self.cSlide_Sw)
        
    def on_btnSlide_Sha_dn_pressed(self):
        self.core.valSw = cSlide_ShiftValue(self,  -1,  self.core.valSw,  self.lcdSw, self.cSlide_Sw)
        
    def on_btnSlide_Sha_up_pressed(self):
        self.core.valSw = cSlide_ShiftValue(self,   1,  self.core.valSw,  self.lcdSw, self.cSlide_Sw)


   
    #######################################
    #   Level Applys
    #######################################
    
    @pyqtSignature("int")
    def on_cSlide_Co_valueChanged(self, value):
        if (self.core.valCo <> value):
            self.core.valCo = cSlide_SetValue(self, value,  self.core.valCo,  self.lcdCo, self.cSlide_Co)
        
    def on_btnSlide_Co_dn_pressed(self):
        self.core.valCo = cSlide_ShiftValue(self,  -1,  self.core.valCo,  self.lcdCo, self.cSlide_Co)
        
    def on_btnSlide_Co_up_pressed(self):
        self.core.valCo = cSlide_ShiftValue(self,   1,  self.core.valCo,  self.lcdCo, self.cSlide_Co)
    
    
    @pyqtSignature("int")
    def on_cSlide_Sh_valueChanged(self, value):
        if (self.core.valSh <> value):
            self.core.valSh = cSlide_SetValue(self, value,  self.core.valSh,  self.lcdSh, self.cSlide_Sh)
            
    def on_btnSlide_Sh_dn_pressed(self):
        self.core.valSh = cSlide_ShiftValue(self,  -1,  self.core.valSh,  self.lcdSh, self.cSlide_Sh)
        
    def on_btnSlide_sh_up_pressed(self):
        self.core.valSh = cSlide_ShiftValue(self,   1,  self.core.valSh,  self.lcdSh, self.cSlide_Sh)

        
    @pyqtSignature("int")
    def on_cSlide_Sa_valueChanged(self, value):
        if (self.core.valSa <> value):
            self.core.valSa = cSlide_SetValue(self, value,  self.core.valSa,  self.lcdSa, self.cSlide_Sa)
    
    def on_btnSlide_Sa_up_pressed(self):
        self.core.valSa = cSlide_ShiftValue(self,   1,  self.core.valSa,  self.lcdSa, self.cSlide_Sa)
        
    def on_btnSlide_Sa_dn_pressed(self):
        self.core.valSa = cSlide_ShiftValue(self,  -1,  self.core.valSa,  self.lcdSa, self.cSlide_Sa)
        
    
    @pyqtSignature("int")
    def on_cSlide_Br_valueChanged(self, value):
        if (self.core.valBr <> value):
            self.core.valBr = cSlide_SetValue(self, value,  self.core.valBr,  self.lcdBr, self.cSlide_Br)
    
    def on_btnSlide_Br_dn_pressed(self):
        self.core.valBr = cSlide_ShiftValue(self,  -1,  self.core.valBr,  self.lcdBr, self.cSlide_Br)
        
    def on_btnSlide_Br_up_pressed(self):
        self.core.valBr = cSlide_ShiftValue(self,  1,  self.core.valBr,  self.lcdBr, self.cSlide_Br)
        
        
        
    @pyqtSignature("int")
    def on_dial_valueChanged(self, value):
        if  (self.core.lockUpdating == False):
            self.core.valRot = value
            self.lcdRot_1.setText(str(self.core.valRot))
            RotateThumb(self,  value)
           

    #######################################
    #   UI Reactions
    #######################################

    def on_btnExitCancel_released(self):
        self.tabStrip.setCurrentIndex(self.core.valCurTab)
        self.FramePopup.setGeometry(QtCore.QRect(0, 0, 0, 0))
        
    def on_btnPopOk_released(self):
        sys.exit()
    
    def on_btnPopCancel_released(self):
        self.tabStrip.setCurrentIndex(self.core.valCurTab)
        self.FramePopup.setGeometry(QtCore.QRect(0, 0, 0, 0))

    def on_minMax_released(self):
        return MinMax()

    def on_photo_released(self):
        QMessageBox.information(self, "Loading Data", "Please wait while the data is retrieved...")

    @pyqtSignature("int")
    def on_tabStrip_currentChanged(self, index):
        """
        Slot documentation goes here.
        """
        if (index==9):
            self.FramePopup.setGeometry(QtCore.QRect(0, 0, 800, 480))
        elif  (index==6):
            ReturnHistPlots(self)
        elif (index==7):
            self.PopulateEXIFData(self.core.curImageLoc)
        else:
            self.core.valCurTab = index
    

    def on_btnOpen_released(self):
        return FileOpenReleased(self)
       
    @pyqtSignature("Image, String")
    def PopulateEXIFData(self,  filename):
        return ImageExifLoad(self)
        
    @pyqtSignature("int")
    def on_tabPhoto_currentChanged(self, index):
        if (index == 2):
            self.btnFullScreenClose.setGeometry(QtCore.QRect(0, 0, 0, 0))
            if self.core.thumbsCount == 0:
                return populatePhotoList(self)
        elif (index ==1):
           self.loadUndoImages()
        else:
            self.btnFullScreenClose.setGeometry(QtCore.QRect(0, 90, 800, 371))
            
    def loadUndoImages(self):
        self.btnUndoCancel.setVisible(False)
        self.btnUndoGo.setVisible(False)
        i = 0
        self.listUndoList.clear()
        for uIm in self.core.undoList_IM:
            icon =QtGui.QIcon(pil2qpixmap(uIm.thumb))
            if (i == 0):
                self.listUndoList.addItem("Original")
            else:
                self.listUndoList.addItem("Step: "+str(i))
                
            self.listUndoList.item(i).setIcon(icon)
            i+=1
            

    #######################################
    #  Full Screen
    #######################################

    def on_btnFullScreen_released(self):
        """
        Button handler for fullscreen  open, calls FullScreenToggle
        """
        self.FullScreenToggle(True)
    

    def on_btnFullScreenClose_released(self):
        """
        Button handler for fullscreen  close, calls FullScreenToggle
        """
        self.FullScreenToggle(False)


    def FullScreenToggle(self,  Show,  ForceTab=0):
        """
        Slot documentation goes here.
        """
        if (Show):
            self.tabPhoto.setCurrentIndex(ForceTab)
            self.framePhoto.setGeometry(QtCore.QRect(0, 0, 800, 480))    
            self.photo.repaint()
            #self.photo_Orig.repaint()
        else:
            self.framePhoto.setGeometry(QtCore.QRect(0, 0, 0, 0))
            
        self.tabPhoto.updateGeometry()
        self.tabStrip.repaint()
        self.repaint()
    
    #######################################
    #   B & W Settings
    #######################################
    def on_btn_Grey_released(self):
        self.core.valBW = "L"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)
    
    def on_btn_Red_released(self):
        self.core.valBW = "R"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)

    def on_btn_Green_released(self):
        self.core.valBW = "G"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)
    
    def on_btn_Blue_released(self):
        self.core.valBW = "B"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)
 
    def on_btn_Yellow_released(self):
        self.core.valBW = "Y"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)
    
    def on_btn_Cyan_released(self):
        self.core.valBW = "C"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)
    
    def on_btn_Magenta_released(self):
        self.core.valBW = "M"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)
        
    def on_btn_BC_released(self):
        self.core.valBW = "BC"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)
    
    def on_btn_GC_released(self):
        self.core.valBW = "GC"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)
    
    def on_btn_GY_released(self):
        self.core.valBW = "GY"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)
    
    def on_btn_BM_released(self):
        self.core.valBW = "BM"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)
        
    def on_btn_RM_released(self):
        self.core.valBW = "RM"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)
    
    def on_btn_RY_released(self):
        self.core.valBW = "RY"
        Convert_BW(self, self.core.current.thumb,  self.core.valBW,  False)


    def on_btnBug_released(self):
        """
        Submit bug report to bugs.maemo.org
        """ 
        url='https://bugs.maemo.org/enter_bug.cgi?product=AnselA'
        os.system('browser --url=%s' %url)


    #######################################
    #   Photo Browser
    #######################################

    def on_btnPListCancel_released(self):
        self.frameCancelPListLoad.setGeometry(QtCore.QRect(0, 0, 0, 0))
        self.core.populatePhotoList = False

    def on_btnPlistZoom_released(self):
        LoadCurImageFromPhotoBrowser(self)
  
    def ToggleThumbWait(self,  Show):
        if Show:
            self.labWaitThumb.setGeometry(-30, -30, 380, 450)
        else:
            self.labWaitThumb.setGeometry(0, 0, 0, 0)

    def on_butReturn_released(self):
        self.FullScreenToggle(False)
    
    def on_listPhotos_itemDoubleClicked(self, item):
        LoadCurImageFromPhotoBrowser(self)
    
    def on_btnReloadPList_released(self):
        self.frameReloadRequest.setGeometry(QtCore.QRect(0, 0, 800, 480))
    
    def on_btnReloadCancel_released(self):
        self.frameReloadRequest.setGeometry(QtCore.QRect(0, 0, 0, 0))
    
    def on_btnReloadGo_released(self):
        self.btnEditZoomed.setGeometry(QtCore.QRect(0, 0, 0, 0))
        self.frameReloadRequest.setGeometry(QtCore.QRect(0, 0, 0, 0))
        populatePhotoList(self,  True)
  
    def on_btnRefresh_pressed(self):
        populatePhotoList(self)

    def on_btnPhotoBrowser_released(self):
        self.FullScreenToggle(True,  2)
        
        if self.core.thumbsCount == 0:
            self.populatePhotoList(self,  True)
    
    def on_listPhotos_itemClicked(self, item):
       #return listPhotosClicked()
        if str(item.text()) == "Load More...":
            self.core.thumbsCount +=100
            populatePhotoList(self,  False)
            self.btnEditZoomed.setGeometry(QtCore.QRect(40, 120, 250, 150))
        else:
            self.listPhotositemLoad(item)
            
    def listPhotositemLoad(self, item):
        if (item <> None):
            self.photoListZoom.setGeometry(item.getIconZoomSize())
            self.btnPlistZoom.setGeometry(self.photoListZoom.geometry())
            self.core.plistIm= item.getImageLocation()
            im = Image.open(self.core.plistIm)
            wTh,  hTh = convertImSize(self.core, im.size[0], im.size[1],  250,  150) #This should be the sdame size as the thumbnailbnails

            plistTim=im.resize((hTh,  wTh))
            self.photoListZoom.setPixmap(pil2qpixmap(plistTim))
            self.photoListZoom.repaint()
        

    def on_listUndoList_itemPressed(self, item):
        self.btnUndoCancel.setVisible(True)
        self.btnUndoGo.setVisible(True)
        if (str(item.text()) == "Original"):
            self.core.undoSelected = 0
        else:
            self.core.undoSelected = str(item.text().replace("Step: ",""))
        self.btnUndoGo.setText("Revert to "+str(item.text()))


    @pyqtSignature("int, int, int, int")
    def ScaleLabel(self, yH, xW, h, w):
        """
        Send widget x(Width), y(Height) of center point
        and h,w of image passed.
        """
        return QtCore.QRect(xW-int(w/2), yH-int(h/2), w, h)
    
    #######################################
    #   Apply and Undo
    #######################################
    def on_btnApply_released(self):
        """
        Apply based on tab selection
        """
        ApplyGo(self)
    

    def on_btnUndoGo_released(self):
        self.core.UndoImage_Set()
        self.loadUndoImages()
        
        self.ToggleThumbWait(True)
        self.FullScreenToggle(False)
        FileImageRefresh(self)
        
    #######################################
    #   Rotation
    #######################################    
    def on_btn_rot_Vert_released(self):
        self.core.flipRight = not self.core.flipRight 
        FlipThumb(self)
        
    def on_btn_rot_Horz_pressed(self):
        self.core.flipBottom = not self.core.flipBottom 
        FlipThumb(self)
        
    def on_btn_rot_90cw_released(self):
        self.core.valRot = 270
        RotateThumb(self,  self.core.valRot)

    def on_btn_rot_90ccw_released(self):
        self.core.valRot = 90
        RotateThumb(self,  self.core.valRot)
        
    def on_btn_rot_180_released(self):
        self.core.valRot = 180
        RotateThumb(self,  self.core.valRot)
    
    
    
    
    #######################################
    #   Save As
    #######################################
    def on_btnSaveAs_released(self):
        self.DisplaySaveAs()


    def on_btnSaveAsCancel_pressed(self):
        self.frameSaveAs.setGeometry(0, 40, 0, 480)
    
        
    def on_btnSaveAsGo_pressed(self):
        self.WaitOverlay("Saving File...")
        self.frameSaveAs.setGeometry(0, 40, 0, 480)
        try:
            self.core.current.image.save(self.btnSaveAsFilenameString().replace("JPEG", "jpg"), str(self.btnSaveAs_Combo.currentText()))
        except IOError:
            self.WaitOverlayOk("Error: Could Not Write File")
        except:
            self.WaitOverlayOk("Error: Failed to Save")
        finally:
            self.WaitOverlayOk("Save Completed")
        
        
        
    def btnSaveAsFilenameString(self):
        return str(self.btnSaveAs_Location.text())+"/"+str(self.btnSaveAs_Filename.text())+"."+str(self.btnSaveAs_Combo.currentText())
        
    def btnSaveAsfindFileExists(self):
        if os.path.isfile(self.btnSaveAsFilenameString()):
            self.btnSaveAsGo.setEnabled(False)
        else:
            self.btnSaveAsGo.setEnabled(True)
            
    def btnSaveAsGenFilename(self, Dir, File):
        i=1
        while os.path.isfile(Dir+File+"_"+str(i)):
            i +=1
        return i
        
    def on_btnSaveAs_Combo_editTextChanged(self, p0):
        self.btnSaveAsfindFileExists()
        
    def on_btnSaveAs_Location_textChanged(self, p0):
        self.btnSaveAsfindFileExists()
        
    def on_btnSaveAs_Filename_textChanged(self, p0):
        self.btnSaveAsfindFileExists()
    
    def on_btnSaveAsCancelOut_pressed(self):
        self.frameSaveAs.setGeometry(0, 40, 0, 480)
        
    def DisplaySaveAs(self):
        fileLoc=StripFilename(self, self.core.curImageLoc)
        i = str(self.btnSaveAsGenFilename(fileLoc[0], fileLoc[1]))
        self.btnSaveAs_Filename.setText(fileLoc[1]+"_"+i)
        self.btnSaveAs_Location.setText(fileLoc[0])
        self.titlePop_Name_3.setText(fileLoc[0]+"/"+fileLoc[1]+"_"+i+"."+str(self.btnSaveAs_Combo.currentText()).replace("JPEG", "jpg"))
        self.frameSaveAuto.setVisible(True)
        self.frameSaveAvd.setVisible(False)
        self.frameSaveAs.setGeometry(0, 40, 800, 480)


    def on_btnSaveAsMore_pressed(self):
        if (str(self.btnSaveAsMore.text()) == "More"):
            self.btnSaveAsMore.setText("Less")
            self.frameSaveAuto.setVisible(False)
            self.frameSaveAvd.setVisible(True)
        else:
            self.btnSaveAsMore.setText("More")
            self.frameSaveAvd.setVisible(False)
            self.frameSaveAuto.setVisible(True)


    #######################################
    #   Pop Wait
    #######################################
    def on_btnPopWaitOk_pressed(self):
        self.WaitOverlayHide()
        
    def WaitOverlay(self,  WaitText, WaitInfoText=""):
        self.labPopWaitInfo.setText(WaitInfoText)
        self.labPopWait.setText(WaitText)
        self.labPopWait.setVisible(True)
        self.FramePopupWait.setVisible(True)
        self.FramePopupWait.setGeometry(0, 0, 800, 480)
        self.btnPopWaitOk.setVisible(False)
        
    def WaitOverlayOk(self,  WaitText, WaitInfoText=""):
        self.labPopWaitInfo.setText(WaitInfoText)
        self.FramePopupWait.setVisible(True)
        self.labPopWait.setText(WaitText)
        self.btnPopWaitOk.setVisible(True)
    
    def WaitOverlayHide(self):
        self.FramePopupWait.setGeometry(0, 0, 0, 480)
    
    def on_btnApplyUndo_pressed(self):
            self.FullScreenToggle(True,  1)
    
    def on_btnEdit_pressed(self):
        LoadCurImageFromPhotoBrowser(self)
    
    def on_btnRefreshExp_pressed(self):
        if (str(self.btnRefreshExp.text()) =="+"):
            self.btnRefreshExp.setText("-")
            self.btnReloadPList.setGeometry(QtCore.QRect(400, 370, 111, 51))
            self.btnOpen.setGeometry(QtCore.QRect(510, 370, 111, 51))
            self.btnRefresh.setGeometry(QtCore.QRect(250, 370, 151, 51))
        else:
            self.btnRefreshExp.setText("+")
            self.btnReloadPList.setGeometry(QtCore.QRect(400, 370, 111, 0))
            self.btnOpen.setGeometry(QtCore.QRect(510, 370, 111, 0))
            self.btnRefresh.setGeometry(QtCore.QRect(250, 370, 151, 0))
    
    
